﻿using Microsoft.EntityFrameworkCore;
using MsBox.Avalonia.Enums;
using MsBox.Avalonia;
using ReactiveUI;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WorkWithDatabase.Models;

namespace WorkWithDatabase.ViewModels
{
    internal class MainPageVM : ViewModelBase
    {
        #region Вывод, добавление, удаление и редактирование преподавателя на главной странице

        List<Teacher> teachersList;

        public List<Teacher> TeachersList { get => teachersList; set => this.RaiseAndSetIfChanged(ref teachersList, value); }

        public MainPageVM()
        {
            TeachersList = MainWindowViewModel.myConnection.Teachers.
                                                            Include(x => x.GenderNavigation).
                                                            Include(x => x.TeacherCourses).ThenInclude(x => x.CourseNavigation).
                                                            Include(x => x.TeacherSubjects).ThenInclude(x => x.SubjectNavigation).ToList();
        }


        public void ToPageAddTeacher()
        {
            MainWindowViewModel.Instance.PageContent = new PageAddTeacher();
        }

        public void ToPageEditTeacher(int IdT)
        {
            MainWindowViewModel.Instance.PageContent = new PageAddTeacher(IdT);
        }

        string messege = "";
        public string Messege { get => messege; set => this.RaiseAndSetIfChanged(ref messege, value); }

        public async void DeleteTeacher(int IdT)
        {
            Messege = "Вы уверенны, что хотите удалить данного преподавателя?";
            ButtonResult result = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением об удалении!", Messege, ButtonEnum.YesNo).ShowAsync();
            Teacher deleteTeacher = MainWindowViewModel.myConnection.Teachers.First(x => x.IdTeacher == IdT);

            switch (result)
            {
                case ButtonResult.Yes:
                    {
                        MainWindowViewModel.myConnection.Teachers.Remove(deleteTeacher);
                        MainWindowViewModel.myConnection.SaveChanges();
                        MainWindowViewModel.Instance.PageContent = new MainPage();
                        Messege = "Преподаватель удалён!";
                        ButtonResult result1 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением об удалении!", Messege, ButtonEnum.Ok).ShowAsync();
                        break;
                    }
                case ButtonResult.No:
                    {
                        Messege = "Удаление отменено!";
                        ButtonResult result2 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением об удалении!", Messege, ButtonEnum.Ok).ShowAsync();
                        break;
                    }
            }
        }

        #endregion

        #region Сортировка,поиск и фильтрация

        string _searchTeach;
        public string SearchTeach { get => _searchTeach; set { _searchTeach = value; filtersTeacher(); } }

        bool _chekWomen = false;
        public bool ChekWomen { get => _chekWomen; set { _chekWomen = value; filtersTeacher(); } }

        bool _chekPhone = false;
        public bool ChekPhone { get => _chekPhone; set { _chekPhone = value; filtersTeacher(); } }

        private List<Subject> subjectsListFilter = [new Subject() { IdSubject = 0, TitleSubject = "Все предметы" }, .. MainWindowViewModel.myConnection.Subjects.ToList()];

        public List<Subject> SubjectsListFilter { get => subjectsListFilter; }

        Subject _selectSub = null;
        public Subject SelectSub 
        {
            get 
            {
                if (_selectSub == null) return subjectsListFilter[0]; 
                else
                {
                    return _selectSub;
                }
            } 
            set { _selectSub = value; filtersTeacher(); } 
        }

        bool _isVisibleSort = false; 
        public bool IsVisibleSort { get => _isVisibleSort; set => this.RaiseAndSetIfChanged(ref _isVisibleSort, value); }

        int _selectedSort = 0;
        public int SelectedSort { get => _selectedSort; set { _selectedSort = value; filtersTeacher(); } }

        bool _sortUp = true; 
        public bool SortUp { get => _sortUp; set { this.RaiseAndSetIfChanged(ref _sortUp, value); filtersTeacher(); } } 

        bool _sortDown = false;
        public bool SortDown { get => _sortDown; set { this.RaiseAndSetIfChanged(ref _sortDown, value); filtersTeacher(); } }

        public void filtersTeacher()
        {
            TeachersList = MainWindowViewModel.myConnection.Teachers.
                                                            Include(x => x.GenderNavigation).
                                                            Include(x => x.TeacherCourses).ThenInclude(x => x.CourseNavigation).
                                                            Include(x => x.TeacherSubjects).ThenInclude(x => x.SubjectNavigation).ToList();

            if (!string.IsNullOrEmpty(_searchTeach))
            {
                TeachersList = TeachersList.Where(x => ((x.Name + " " + x.Surname+ " " + x.Patronymic).ToLower().Contains(_searchTeach.ToLower()))).ToList();
            }

            if (_chekWomen)
            {
                TeachersList = TeachersList.Where(x => x.Gender==2).ToList();
            }

            if (_chekPhone)
            {
                TeachersList = TeachersList.Where(x => x.Phone != "").ToList();
            }

            if (SelectSub != null && SelectSub.IdSubject != 0)
            {
                TeachersList = TeachersList.Where(x => x.TeacherSubjects.Any(y => y.SubjectNavigation.IdSubject == _selectSub.IdSubject)).ToList();
            }

            switch (_selectedSort)
            {
                case 0: IsVisibleSort = false; break;
                case 1: IsVisibleSort = true;
                    if (_sortDown) TeachersList = TeachersList.OrderByDescending(x => x.Experience).ToList();
                    else TeachersList = TeachersList.OrderBy(x => x.Experience).ToList();
                break;
                case 2: IsVisibleSort = true; 
                    if (_sortDown) TeachersList = TeachersList.OrderByDescending(x => x.Surname).ToList();
                    else TeachersList = TeachersList.OrderBy(x => x.Surname).ToList();
                break;
                case 3:
                    IsVisibleSort = true;                    
                    if (_sortDown) TeachersList = TeachersList.OrderByDescending(x => x.hoursCourse).ToList();
                    else TeachersList = TeachersList.OrderBy(x => x.hoursCourse).ToList();
                break;
            }
        }

        #endregion
    }
}
