﻿using Avalonia.Controls;
using Avalonia.Media;
using Microsoft.EntityFrameworkCore;
using MsBox.Avalonia;
using MsBox.Avalonia.Enums;
using ReactiveUI;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using WorkWithDatabase.Models;
using WorkWithDatabase.Views;

namespace WorkWithDatabase.ViewModels
{
    internal class PageAddTeacherVM: ViewModelBase
    {
        public void ToMainPage()
        {
            MainWindowViewModel.Instance.PageContent = new MainPage();
        }

        public void ShowAddCourse()
        {
            var win = new Window
            {
                Width = 400,
                Height = 300,
                Content = new PageAddCourse(),
                WindowStartupLocation = WindowStartupLocation.CenterOwner
            };
            win.Show();
        }

        Teacher? _newTeaher;
        public Teacher? NewTeaher { get => _newTeaher; set => this.RaiseAndSetIfChanged(ref _newTeaher, value); }

        string _BtAddSaveChange;
        public string BtAddSaveChange { get => _BtAddSaveChange; set => this.RaiseAndSetIfChanged(ref _BtAddSaveChange, value); }

        string messege = "";
        public string Messege { get => messege; set => this.RaiseAndSetIfChanged(ref messege, value); }

        Course? _newCourse;

        public Course? NewCourse { get => _newCourse; set => this.RaiseAndSetIfChanged(ref _newCourse, value); }
        public PageAddTeacherVM()
        {
            BtAddSaveChange = "Добавить";
            _newTeaher = new Teacher() { GenderNavigation = new Gender() };
            _newCourse = new Course();
        }
        public PageAddTeacherVM(int IdT)
        {
            BtAddSaveChange = "Сохранить изменения";
            _newCourse = new Course();
            _newTeaher = MainWindowViewModel.myConnection.Teachers.
                Include(x => x.GenderNavigation).
                Include(x => x.TeacherCourses).ThenInclude(x => x.CourseNavigation).
                Include(x => x.TeacherSubjects).ThenInclude(x => x.SubjectNavigation).
                FirstOrDefault(x => x.IdTeacher == IdT);
            CourseTable = NewTeaher.TeacherCourses.ToList();
            SubjectTable = NewTeaher.TeacherSubjects.ToList();
        }

        public void ClosePageCourse()
        {
            MainWindowViewModel.Instance.PageContent = new PageAddTeacher();
        }
        public async void SaveCourse()
        {
            try
            {
                if (NewCourse.TitleCourse == null && NewCourse.Hours == null)
                {
                    Messege = "Все поля должны быть заполнены!";
                    ButtonResult result1 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о некорректных данных!", Messege, ButtonEnum.Ok).ShowAsync();
                    
                }
                else if (NewCourse.TitleCourse != null && NewCourse.Hours == null)
                {
                    Messege = "В поле с часами должно быть только целое число!";
                    ButtonResult result2 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о некорректных данных!", Messege, ButtonEnum.Ok).ShowAsync();
                }
                else
                {
                    if (NewCourse.IdCourse == 0)
                    {
                        MainWindowViewModel.myConnection.Courses.Add(NewCourse);
                    }
                    MainWindowViewModel.myConnection.SaveChanges();
                    MainWindowViewModel.Instance.PageContent = new PageAddTeacher();
                    Messege = "Курс добавлен!";
                    ButtonResult result2 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о добавлении!", Messege, ButtonEnum.Ok).ShowAsync();
                }
            }
            catch (Exception ex)
            {                
                           
                MessageBoxManager.GetMessageBoxStandard("Внимание", ex.Message + "\n" + ex.StackTrace, MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
            }
        }

        public List<Gender> GendersList => MainWindowViewModel.myConnection.Genders.ToList();      

        // свойство, хранящее список предметов (причем, если объект редактируется, то из этого списка с помощью Except уже будут удалены все выбранные ранее предметы учителя)
        public List<Subject> SubjectsList => MainWindowViewModel.myConnection.Subjects.ToList().Except(_newTeaher.TeacherSubjects.Select(x => x.SubjectNavigation)).ToList();

        List<TeacherSubject>? _subjectTable; // список предметов учителя, которые уже были ему выбраны

        // свойство для обращения к полю, хранящему список предметов
        public List<TeacherSubject>? SubjectTable { get => _subjectTable; set => this.RaiseAndSetIfChanged(ref _subjectTable, value); }

        public Subject? SelectedSubject
        {
            get => null;            
            set
            {
                if (value != null)
                {
                    NewTeaher.TeacherSubjects.Add(new TeacherSubject() { TeacherNavigation = NewTeaher, SubjectNavigation = value });
                    SubjectTable = NewTeaher.TeacherSubjects.ToList();
                    this.RaisePropertyChanged(nameof(SubjectsList));
                }
            }
        }

        public List<Course> CoursesList => MainWindowViewModel.myConnection.Courses.ToList().Except(_newTeaher.TeacherCourses.Select(x => x.CourseNavigation)).ToList();

        List<TeacherCourse>? _courseTable;
        public List<TeacherCourse>? CourseTable { get => _courseTable; set => this.RaiseAndSetIfChanged(ref _courseTable, value); }

        public Course? SelectedCourse 
        {
            get => null;
            set
            {
                if (value != null)
                {
                    NewTeaher.TeacherCourses.Add(new TeacherCourse() { TeacherNavigation = NewTeaher, CourseNavigation = value });
                    CourseTable = NewTeaher.TeacherCourses.ToList();
                    this.RaisePropertyChanged(nameof(CoursesList));
                }
            }
        }               

        public void DeleteCourses(TeacherCourse deleteCourse)
        {
            try
            {
                NewTeaher.TeacherCourses.Remove(deleteCourse);
                CourseTable = NewTeaher.TeacherCourses.ToList();
                this.RaisePropertyChanged(nameof(CoursesList));
            }
            catch (Exception ex)
            {
                MessageBoxManager.GetMessageBoxStandard("Внимание", ex.Message + "\n" + ex.StackTrace, MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
            }           
        }
        
        public void DeleteSubjects(TeacherSubject deleteSubject)
        {
            try
            {
                NewTeaher.TeacherSubjects.Remove(deleteSubject);
                SubjectTable = NewTeaher.TeacherSubjects.ToList();
                this.RaisePropertyChanged(nameof(SubjectsList));
            }
            catch (Exception ex)
            {
                MessageBoxManager.GetMessageBoxStandard("Внимание", ex.Message + "\n" + ex.StackTrace, MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
            }           
        }

        public async void SaveAddTeacher()
        {
            try
            {
                if (NewTeaher.Surname == null || NewTeaher.Name == null || NewTeaher.Patronymic == null
                    || NewTeaher.Birthday == null || NewTeaher.Email == null)
                {
                    Messege = "Все поля, кроме телефона, должны быть заполнены!";
                    ButtonResult result1 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о некорректных данных!", Messege, ButtonEnum.Ok).ShowAsync();
                }
                else if (NewTeaher.Surname != null && NewTeaher.Name != null && NewTeaher.Patronymic != null &&
                 NewTeaher.Birthday != null && NewTeaher.Experience == null && NewTeaher.Email != null)
                {
                    Messege = "В поле со стажем должно быть только целое число!";
                    ButtonResult result2 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о некорректных данных!", Messege, ButtonEnum.Ok).ShowAsync();
                }
                else
                {
                    if (NewTeaher.IdTeacher == 0)
                    {
                        MainWindowViewModel.myConnection.Teachers.Add(NewTeaher);
                    }
                    MainWindowViewModel.myConnection.SaveChanges();
                    MainWindowViewModel.Instance.PageContent = new MainPage();
                    if (BtAddSaveChange == "Добавить")
                    {
                        Messege = "Преподаватель добавлен!";
                        ButtonResult result3 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением о добавлении!", Messege, ButtonEnum.Ok).ShowAsync();
                    }
                    else if (BtAddSaveChange == "Сохранить изменения")
                    {
                        Messege = "Изменения о преподователе сохранены!";
                        ButtonResult result4 = await MessageBoxManager.GetMessageBoxStandard("Сообщение с уведомлением об изменениях!", Messege, ButtonEnum.Ok).ShowAsync();
                    }
                }  
            }
            catch(Exception ex)
            {        
                MessageBoxManager.GetMessageBoxStandard("Внимание", ex.Message + "\n" + ex.StackTrace, MsBox.Avalonia.Enums.ButtonEnum.Ok).ShowAsync();
            }            
        }        
    }
}
